#!/usr/bin/env python3

from typing import Dict, List
import requests
import json


API_URL = 'https://management.azure.com'

class azure:

    def get_auth_token(token: str) -> Dict:
        return requests.post(f'https://login.microsoftonline.com/{token["tenant_id"]}/oauth2/token',
                    data={
                        'grant_type':'client_credentials',
                        'client_id': token["client_id"],
                        'client_secret': token["client_secret"],
                        'resource': API_URL
                    }).json()
    

    @staticmethod 
    def authorized(token: str) -> bool:
        resp = azure.get_auth_token(token)
    
        if 'error' in resp:
            return False
    
        return True
    

    def get_offers(oauth_token: str, subscription: str, publisher: str, location: str) -> List:
        offers = list()

        resp = requests.get(f'{API_URL}/subscriptions/{subscription}/providers/Microsoft.Compute/locations/{location}/publishers/{publisher}/artifacttypes/vmimage/offers?api-version=2022-03-01', headers={
                'Authorization': f'Bearer {oauth_token}',
                'Content-Type': 'application/json'
                    }).json()

        for offer in resp:
            try:
                # test-ubuntu-premium-offer-0002
                if offer['name'].startswith('test'):
                    continue

                # 0001-com-ubuntu-confidential-vm-test-focal
                int(offer['name'].split('-')[0])

            except:
                offers.append(offer['name'])

        return offers


    def get_skus(oauth_token: str, subscription: str, publisher: str, location: str, offer: str) -> List:
        resp = requests.get(f'{API_URL}/subscriptions/{subscription}/providers/Microsoft.Compute/locations/{location}/publishers/{publisher}/artifacttypes/vmimage/offers/{offer}/skus?api-version=2022-03-01', headers={
                'Authorization': f'Bearer {oauth_token}',
                'Content-Type': 'application/json'
                    }).json()

        return [sku['name'] for sku in resp]

    
    @staticmethod 
    def get_images(token: str, location='eastus') -> Dict:
        images = dict()

        oauth_token = azure.get_auth_token(token)['access_token']

        for publisher in ['Canonical', 'Debian']:
            # Gets Offers
            offers = azure.get_offers(oauth_token, token['subscription_id'], publisher, location)

            # Query skus
            for offer in offers:
                skus = azure.get_skus(oauth_token, token['subscription_id'], publisher, location, offer)

                # populate images: {publisher}/{offer}/{sku}
                for sku in skus:
                    image = f'{publisher}/{offer}/{sku}'
                    images[image] = f'{offer} {sku}'


        return images
    

    @staticmethod 
    def valid_image_in_region(image: str, region: str, token: str) -> Dict:
        images = azure.get_images(token, region)

        if image in images:
            return True

        return False


    @staticmethod 
    def get_regions(token: str) -> Dict:
        regions = dict()

        oauth_token = azure.get_auth_token(token)['access_token']
        # Need to figure out pagination, but in sample test it was a single page
        resp = requests.get(f'{API_URL}/subscriptions/{token["subscription_id"]}/locations?api-version=2022-01-01', headers={
                    'Authorization': f'Bearer {oauth_token}',
                    'Content-Type': 'application/json'
                        }).json()

        for region in resp['value']:
            try:
                regions[region['name']] = region['displayName']
            except:
                pass


        return regions



#!/usr/bin/env python3
# -*- coding:utf-8 -*-
'''
Terraform workspace helper functions
'''
from typing import List
import python_terraform
import re


def get_current_workspace(tf: python_terraform.Terraform) -> str:
    ret, stdout, stderr = tf.cmd("workspace", "show")
    if ret != 0 and stderr:
        stderr = re.sub(f'(\[\d+m)', r'\033\1', stderr)
        print(stderr)

    return stdout.strip()


def get_all_workspace(tf: python_terraform.Terraform) -> List[str]:
    ret, stdout, stderr = tf.cmd("workspace", "list")
    if ret != 0 and stderr:
        stderr = re.sub(f'(\[\d+m)', r'\033\1', stderr)
        print(stderr)
    
    return list(map(lambda s: s.strip('* ').strip(), stdout.split('\n')))


def create_new_workspace(tf: python_terraform.Terraform, name: str) -> bool:
    ret, stdout, stderr = tf.cmd("workspace", "new", name)
    if ret != 0 and stderr:
        stderr = re.sub(f'(\[\d+m)', r'\033\1', stderr)
        print(stderr)

        return False

    return True


def select_workspace(tf: python_terraform.Terraform, name: str) -> bool:
    ret, stdout, stderr = tf.cmd("workspace", "select", name)
    if ret != 0 and stderr:
        stderr = re.sub(f'(\[\d+m)', r'\033\1', stderr)
        print(stderr)

        return False

    return True



#!/usr/bin/env python3
# -*- coding:utf-8 -*-

from io import BytesIO
from urllib.request import urlopen
from zipfile import ZipFile
from typing import Dict, List
from rich.table import Table
import jsonschema
import platform
import hashlib
import random
import string
import shutil
import json
import yaml
import stat
import os

from tempor import provider_info, ROOT_DIR, CONFIG_DIR, BIN_DIR, DATA_DIR
from tempor.console import console
from tempor.ssh import remove_config_entry

TF_VER = "1.1.9"
TF_ZIP_HASH = {
    "amd64": "9d2d8a89f5cc8bc1c06cb6f34ce76ec4b99184b07eb776f8b39183b513d7798a",
    "386": "a29a5c069e1712753ed553f7c6e63f1cd35caefee73496210461c05158b836b4",
    "arm": "800eee18651b5e552772c60fc1b5eb00cdcefddf11969412203c6de6189aa10a",
    "arm64": "e8a09d1fe5a68ed75e5fabe26c609ad12a7e459002dea6543f1084993b87a266",
    "darwin": "c902b3c12042ac1d950637c2dd72ff19139519658f69290b310f1a5924586286",
}
TF_FILE_HASH = {
    "amd64": "8d5b3b0a164e95de9cafbdd6ca16a1ec439927b9bb6ec146b9566473ca796cc0",
    "386": "db94691af978caa67b2c6a527d46cf8c7ea738c42a3750a121ddf4eb993bab25",
    "arm": "4c797f48f7614706e35ecd60e16f0c776404515119d37eed44c0417194a0426b",
    "arm64": "d501a25b7f95dfa3d5414bc4fc5382c09fe926464c4114a288ddbd7bb688d94c",
    "darwin": "41ea760fa6b4b60525731af0acda64e76cc21f098a6f33b7c92868f5c8667a7f",
}

HOSTS_FILE = f"{DATA_DIR}/hosts"
ANSIBLE_HOSTS = f"{ROOT_DIR}/playbooks/inventory"


def image_region_choices(provider: str) -> str:
    if provider is None:
        print(''' ''')
        return

    reg_table = Table(title="Regions")
    reg_table.add_column("ID", style="cyan")
    if provider == 'gcp':
        reg_table.add_column("Zones", style="magenta")
    else:
        reg_table.add_column("Location", style="magenta")

    for _id,name in provider_info[provider]['regions'].items():
        reg_table.add_row(str(_id), str(name))

    img_table = Table(title="Images x86-64")
    img_table.add_column("ID", style="cyan")
    img_table.add_column("Name", style="magenta")
    for _id,name in provider_info[provider]['images'].items():
        img_table.add_row(str(_id), str(name))

    print(f'''
usage: tempor {provider} [-h] [--image image] [--region region] [-s] [-l] [-b] [-m] [--teardown]

options:
  -h, --help       show this help message and exit
  --image image    Specify the OS Image
  --region region  Specify the Region to Host the Image
  -s, --setup      Create VPS'
  -l, --list       List Available VPS'
  -b, --bare       Leave as a Bare Install
  -m, --minimal    Minimal Configuration
  --teardown       Tear down VPS'
''')
    console.print(reg_table)
    console.print(img_table)


def get_config() -> Dict:
    fpath = f"{CONFIG_DIR}/config.yml"

    if not os.path.exists(fpath):
        console.print(f"Creating New Config File: {fpath}")
        shutil.copy(f"{ROOT_DIR}/config/config.yml", fpath)
        return None

    with open(fpath) as fr, open(f"{ROOT_DIR}/config/schema.yml") as fr2:
        try:
            cfg = yaml.safe_load(fr)
            schema = yaml.safe_load(fr2)
            jsonschema.validate(cfg, schema)
        except jsonschema.exceptions.ValidationError as e:
            console.print(f"[red bold]Invalid Config File: {fpath}")
            if "api_token" in str(e):
                console.print(
                    "[red bold]All Values are required. Remove Providers without an API Token"
                )
            else:
                console.print(f"[red bold]{e}")
            return None
        return cfg


def terraform_installed() -> str:
    updated = True
    out_file = shutil.which("terraform")

    uname = platform.uname()
    if "linux" in uname.system.lower():
        if "aarch64" in uname.machine:
            arch = "arm64"
        elif "64" in uname.machine:
            arch = "amd64"
        elif "386" in uname.machine:
            arch = "386"
        else:
            arch = "arm"
        url = f"https://releases.hashicorp.com/terraform/{TF_VER}/terraform_{TF_VER}_linux_{arch}.zip"
    elif "darwin" in uname.system.lower():
        arch = "darwin"
        url = f"https://releases.hashicorp.com/terraform/{TF_VER}/terraform_{TF_VER}_darwin_amd64.zip"
    else:
        return None

    # Check if we've already installed
    if not out_file:
        out_file = f"{BIN_DIR}/terraform"
    else:  # check version
        h = hashlib.sha256()
        with open(out_file, 'rb') as fr:
            tf = BytesIO(fr.read())

            if TF_FILE_HASH[arch] != hashlib.sha256(tf.getvalue()).hexdigest():
                updated = False


    if not os.path.exists(out_file) or not updated:
        out_file = f"{BIN_DIR}/terraform"
        console.print(f"Terraform not in Path or Out-of-Date. Installing v{TF_VER} to {out_file} ...")

        h = hashlib.sha256()
        with urlopen(url) as zipresp:
            zipfile = BytesIO(zipresp.read())

            console.print(f"Validating Hash: {TF_ZIP_HASH[arch]}")
            assert (
                TF_ZIP_HASH[arch] == hashlib.sha256(zipfile.getvalue()).hexdigest()
            ), "Invalid SHA256 Hash of Zip File!"
            console.print("Passed!")
            with ZipFile(zipfile) as zfile:
                zfile.extractall(f"{BIN_DIR}")
            st = os.stat(out_file)
            os.chmod(out_file, st.st_mode | stat.S_IXUSR)
    return out_file


def rm_hosts(provider: str, workspace: str) -> None:
    hostnames = list()
    hosts = get_hosts()

    if not hosts or provider not in hosts:
        return

    for idx, host in enumerate(hosts[provider]):
        for hostname, values in host.items():
            if values['workspace'] == workspace:
                hostnames.append(hostname)
                del hosts[provider][idx]
                remove_config_entry(hostname)


    with open(HOSTS_FILE, "w") as fw:
        json.dump(hosts, fw)

    if os.path.exists(ANSIBLE_HOSTS):
        with open(ANSIBLE_HOSTS) as fr, open(ANSIBLE_HOSTS, "w+") as fw:
            for line in fr:
                for hostname in hostnames:
                    line = line.replace(hostname, '')
                fw.write(line)


def get_hosts() -> Dict:
    hosts = dict()
    try:
        if os.path.exists(HOSTS_FILE):
            with open(HOSTS_FILE) as fr:
                hosts = json.load(fr)
    except json.decoder.JSONDecodeError:
        pass
    return hosts


def find_hostname(name: str) -> str:
    all_hosts = get_hosts()

    for provider in all_hosts:
        for idx, vps in enumerate(all_hosts[provider]):
            if name in vps:
                return all_hosts[provider][idx][name]


'''
{
    '<provider>': [
        '<hostname>': {
            'ip': '<ip>',
            'workspace': '<workspace>'
        },
    ]
}
'''
def save_hosts(provider: str, new_hosts: dict) -> None:
    hosts = dict()

    if not isinstance(new_hosts, dict):
        console.print("[red bold]Cannot save new hosts")
        return

    # load in what's there
    hosts = get_hosts()

    # combine 2 dictionaries
    if provider not in hosts:
        hosts[provider] = list()

    hosts[provider].append(new_hosts)

    with open(HOSTS_FILE, "w") as fw:
        json.dump(hosts, fw)

    with open(ANSIBLE_HOSTS, "a+") as fw:
        for host in hosts[provider]:
            for hostname, values in host.items():
                fw.write(f"{hostname}\n")


def random_line(f: str) -> str:
    with open(f) as fr:
        lines = fr.read().splitlines()
        return random.choice(lines).replace("'", "").lower()


def random_number(min_val: int = 0, max_val: int = 100) -> int:
    return str(random.randint(min_val, max_val))


def random_str(min_val: int = 5, max_val: int = 10) -> str:
    return "".join(
        [
            random.choice(string.ascii_lowercase)
            for _ in range(random.randint(min_val, max_val))
        ]
    )


def random_name() -> str:
    try:
        wordlist = Path("/usr/share/dict/american-english")
        if not wordlist.is_file():
            raise FileNotFoundError
        return f"{random_line(wordlist)}{random_number()}"
    except FileNotFoundError:
        return f"{random_str()}{random_number()}"



#!/usr/bin/env python
# -*- coding:utf-8 -*-
import string
import secrets
import json

alphabet = string.ascii_letters + string.digits
while True:
    password = "".join(secrets.choice(alphabet) for i in range(32))
    if (
        any(c.islower() for c in password)
        and any(c.isupper() for c in password)
        and sum(c.isdigit() for c in password) >= 3
    ):
        break
result = {"value": password}
print(json.dumps(result))



#!/usr/bin/env python3
# -*- coding:utf-8 -*-

import pytest

from tempor.utils import random_number

def test_random_number():
    assert random_number().isnumeric()

def test_raise_exception_on_invalid_random_number():
    with pytest.raises(Exception):
        random_number('j')




#!/usr/bin/env python3

from typing import Dict
import requests
import json


API_URL = 'https://api.digitalocean.com/v2'

class digitalocean:

    def get_account(token: str) -> Dict:
        return requests.get(f'{API_URL}/account', headers={
                    'Authorization': f'Bearer {token}',
                    'Content-Type': 'application/json'
                        }).json()


    @staticmethod 
    def authorized(token: str) -> bool:
        resp = digitalocean.get_account(token)
    
        if 'id' in resp and resp['id'] == 'Unauthorized':
            return False
    
        return True
    
    
    @staticmethod 
    def get_images(token: str) -> Dict:
        images = dict()

        page = 1
        while True:
            resp = requests.get(f'{API_URL}/images?per_page=500&page={page}', headers={
                    'Authorization': f'Bearer {token}',
                    'Content-Type': 'application/json'
                        }).json()


            for image in resp['images']:
                images[image['slug']] = image['name']

            if not resp['links'] or 'last' not in resp['links']['pages']:
                break

            page += 1

        return images
    

    @staticmethod 
    def valid_image_in_region(image: str, region: str, token: str) -> Dict:
        images = list()

        page = 1
        while True:
            resp = requests.get(f'{API_URL}/images?per_page=500&page={page}', headers={
                    'Authorization': f'Bearer {token}',
                    'Content-Type': 'application/json'
                        }).json()


            for i in resp['images']:
                if i['slug'] == image:
                    # image is in region and available
                    return region in i['regions'] and i['status'] == 'available'

            if not resp['links'] or 'last' not in resp['links']['pages']:
                break

            page += 1

        return False


    @staticmethod 
    def get_regions(token: str) -> Dict:
        regions = dict()

        page = 1
        while True:
            resp = requests.get(f'{API_URL}/regions?per_page=500&page={page}', headers={
                    'Authorization': f'Bearer {token}',
                    'Content-Type': 'application/json'
                        }).json()

            for region in resp['regions']:
                if region['available'] == True:
                    regions[region['slug']] = region['name']

            if not resp['links'] or 'last' not in resp['links']['pages']:
                break

            page += 1

        return regions



#!/usr/bin/env python3

from googleapiclient import discovery
import google.auth

from typing import List, Dict


class gcp:

    @staticmethod 
    def authorized(api_token: dict) -> bool:
        try:
            auth_file = api_token['auth_file']

            google.auth.load_credentials_from_file(auth_file)

            return True
        except Exception as e:
            print(e)
            return False
    
    
    @staticmethod 
    def get_images(api_token: dict) -> Dict:
        #return {
        #    'ubuntu-os-cloud/ubuntu-2004-lts': 'Ubuntu 20.04',
        #    'ubuntu-os-cloud/ubuntu-1804-lts': 'Ubuntu 18.04'
        #}
        images = dict()

        auth_file = api_token['auth_file']
        proj = api_token['project']

        creds, _ = google.auth.load_credentials_from_file(auth_file)
        service = discovery.build('compute', 'v1', credentials=creds)

        # this is stupid see https://issuetracker.google.com/issues/64718267?pli=1
        # https://cloud.google.com/compute/docs/images/os-details
        projects = [
            proj,
            'centos-cloud',
            'debian-cloud',
            'fedora-coreos-cloud',
            'rocky-linux-cloud',
            'ubuntu-os-cloud',
            'ubuntu-os-pro-cloud',
        ]

        for project in projects:
            resp = service.images().list(project=project).execute()
            
            if 'items' not in resp:
                continue
            
            for image in resp['items']:
                # ignore the deprecated and obsolete images
                if 'deprecated' in image and \
                        'state' in image['deprecated'] and \
                        image['deprecated']['state'] in ['OBSOLETE', 'DEPRECATED']:
                            continue

                '''
                The image from which to initialize this disk. 
                This can be one of: the image's self_link, 
                projects/{project}/global/images/{image}, 
                projects/{project}/global/images/family/{family}, 
                global/images/{image}, global/images/family/{family}, 
                family/{family}, {project}/{family}, 
                {project}/{image}, {family}, or {image}
                '''
                try:
                    _image = f'{project}/{image["family"]}'
                    images[_image] = image['description']
                except:
                    _image = f'{project}/{image["name"]}'
                    images[_image] = image['name']


        return images
    

    @staticmethod 
    def valid_image_in_region(image: str, region: str, token: str) -> Dict:
        return True
    

    @staticmethod 
    def get_regions(api_token: dict) -> List:
        regions = dict()

        auth_file = api_token['auth_file']
        project = api_token['project']

        creds, _ = google.auth.load_credentials_from_file(auth_file)
        service = discovery.build('compute', 'v1', credentials=creds)
        
        resp = service.regions().list(project=project).execute()
        for region in resp['items']:
            if region['status'] == 'UP':
                regions[region['name']] = [z.split('zones/')[-1] for z in region['zones']]

        return regions



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import ansible_runner
import os

from tempor import ROOT_DIR
from tempor.console import console


def run_playbook(playbook: str = "main.yml", user: str = "root") -> None:
    ansible_runner.run(
        private_data_dir=f"{ROOT_DIR}/playbooks", playbook=playbook, verbosity=1, 
        extravars = {"ansible_user": user}
    )



#!/usr/bin/env python3

from typing import Dict
import requests
import json


API_URL = 'https://api.linode.com/v4'

class linode:

    def get_account(token: str) -> Dict:
        return requests.get(f'{API_URL}/account', headers={
                    'Authorization': f'Bearer {token}',
                    'Content-Type': 'application/json'
                        }).json()
    

    @staticmethod 
    def authorized(token: str) -> bool:
        resp = linode.get_account(token)
    
        if 'errors' in resp:
            return False
    
        return True
    
    
    @staticmethod 
    def get_images(token: str) -> Dict:
        images = dict()

        page = 1
        while True:
            resp = requests.get(f'{API_URL}/images?page={page}', headers={
                    'Authorization': f'Bearer {token}',
                    'Content-Type': 'application/json'
                        }).json()
            for image in resp['data']:
                if image['status'] == 'available':
                    images[image['id']] = image['label']


            if page == resp['pages']:
                break
            page += 1

        return images
    

    @staticmethod 
    def valid_image_in_region(image: str, region: str, token: str) -> Dict:
        return True  # linode does not restrict images in regions


    @staticmethod 
    def get_regions(token: str) -> Dict:
        regions = dict()

        page = 1
        while True:
            resp = requests.get(f'{API_URL}/regions?page={page}', headers={
                    'Authorization': f'Bearer {token}',
                    'Content-Type': 'application/json'
                        }).json()

            for region in resp['data']:
                if 'Linodes' in region['capabilities']:
                    regions[region['id']] = region['country']

            if page == resp['pages']:
                break
            page += 1

        return regions



#!/usr/bin/env python3

from boto3.session import Session
from typing import List, Dict
import boto3


class aws:

    @staticmethod 
    def authorized(api_token: dict) -> bool:
        try:
            access_key = api_token['access_key']
            secret_key = api_token['secret_key']

            client = boto3.client(
                'sts',
                aws_access_key_id=access_key,
                aws_secret_access_key=secret_key,
                region_name='us-east-1'
            )
            client.get_caller_identity()
            return True
        except Exception as e:
            print(e)
            return False
    
    
    @staticmethod 
    def get_images(api_token: dict, region: str = 'us-east-1') -> Dict:
        return {
            'ami-0f9fc25dd2506cf6d': 'Amazon Linux 2',
            'ami-0d6e9a57f6259ba3a': 'Centos 8',
            'ami-02358d9f5245918a3': 'Centos 7 ',
            'ami-0d35afd5d19280755': 'Debian 11',
            'ami-059e59467656af55e': 'Debian 10',
            'ami-03f9e5587a7d588f8': 'Debain 9',
            'ami-01691107cfcbce68c': 'Kali',
            'ami-0ec1545979d0dc885': 'Rocky 8',
            'ami-04505e74c0741db8d': 'Ubuntu 20.04',
            'ami-0e472ba40eb589f49': 'Ubuntu 18.04',
            'ami-0b0ea68c435eb488d': 'Ubuntu 16.04',
            
        }
        # below is how to query API, but 2k+ items are returned
        images = dict()

        access_key = api_token['access_key']
        secret_key = api_token['secret_key']
        client = boto3.client('ec2', 
                aws_access_key_id=access_key,
                aws_secret_access_key=secret_key,
                region_name=region)
        resp = client.describe_images(
                    Owners=[
                        '099720109477',
                        '679593333241',
                    ],
                    Filters=[
                        {
                            'Name': 'architecture',
                            'Values': ['x86_64']
                        },
                        {
                            'Name': 'state',
                            'Values': ['available']
                        }
                    ], IncludeDeprecated=False)

        # print(resp)
        for image in resp['Images']:
            try:
                images[image['ImageId']] = image['Description']
            except KeyError as e:
                pass

        print(len(images))
        return images
    

    @staticmethod 
    def valid_image_in_region(image: str, region: str, token: str) -> Dict:
        images = aws.get_images(token, region)
        
        if image in images.keys():
            return True

        return False
    

    @staticmethod 
    def get_regions(api_token: dict) -> List:
        regions = dict()

        session = Session()
        resp = session.get_available_regions('ec2') 

        for region in resp:
            regions[region] = region

        return regions



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import ssh_config
from ssh_config import SSHConfig, Host
from os.path import expanduser
from pathlib import Path
from os import path
import subprocess
import shutil
import shlex
import sys
import os

from tempor import ROOT_DIR, DATA_DIR
from tempor.console import console

SSH_CONFIG_PATH = expanduser("~/.ssh/config")


def add_config_entry(hostname: str, attr: dict) -> None:
    new_host = Host(hostname, attr)

    # does ~/.ssh/config exist?
    if not path.isfile(expanduser(SSH_CONFIG_PATH)):
        # ~/.ssh/ ?
        if not path.exists(os.path.dirname(SSH_CONFIG_PATH)):
            os.makedirs(os.path.dirname(SSH_CONFIG_PATH))
        # create ~/.ssh/config
        cfg = SSHConfig(expanduser(SSH_CONFIG_PATH))
    else:
        try:
            cfg = SSHConfig.load(expanduser(SSH_CONFIG_PATH))
        except ssh_config.client.EmptySSHConfig:
            cfg = SSHConfig(expanduser(SSH_CONFIG_PATH))


    cfg.append(new_host)
    cfg.write()


def remove_config_entry(hostname: str) -> None:
    # Nothing to remove if config doesn't exist
    if not path.isfile(expanduser(SSH_CONFIG_PATH)):
        return

    cfg = SSHConfig.load(expanduser(SSH_CONFIG_PATH))

    try:
        cfg.remove(hostname)
        cfg.write()
    except KeyError:
        pass

    try:
        shutil.rmtree(f"{DATA_DIR}/{hostname}")
    except OSError as e:
        pass

    try:
        shutil.rmtree(f"{ROOT_DIR}/playbooks/artifacts")
    except OSError as e:
        pass


def check_sshkeys(provider: str, region: str, image: str) -> bool:
    # azure only allows RSA keys, so dumb
    key_type = "rsa" if provider == "azure" else "ed25519"


    prog = shutil.which("ssh-keygen")
    if not prog:
        console.print("[red bold]ssh-keygen not available. Is OpenSSH installed?")
        return False

    out_dir = f"{ROOT_DIR}/providers/{provider}/files/{region}/{image}/.ssh"
    if not os.path.exists(out_dir):
        os.makedirs(out_dir)

    out_file = f"{out_dir}/id_{key_type}"
    if not os.path.exists(out_file):
        console.print("Generating new key pair...", end="", style="bold italic")
        subprocess.call(
            f'yes | ssh-keygen -t {key_type} -N "" -C "" -f {out_file}',
            stdout=subprocess.DEVNULL,
            shell=True,
        )
        console.print("Done.")


def install_ssh_keys(provider: str, region: str, image: str, hostname: str, ip_address: str, user: str) -> None:
    old_dir = f"{ROOT_DIR}/providers/{provider}/files/{region}/{image}/.ssh"
    out_dir = f"{DATA_DIR}/{hostname}/ssh"
    if not os.path.exists(out_dir):
        os.makedirs(out_dir)

    for fname in os.listdir(old_dir):
        shutil.copy(os.path.join(old_dir, fname), out_dir)

    attr = {
        "Hostname": ip_address,
        "User": user,
        "Port": 22,
        "Compression": "yes",
        "StrictHostKeyChecking": "no",
        "UserKnownHostsFile": "/dev/null",
        "IdentityFile": f"{out_dir}/id_ed25519",
    }
    add_config_entry(hostname, attr)



#!/usr/bin/env python3
# -*- coding:utf-8 -*-

from tempor.apis.aws import aws
from tempor.apis.azure import azure
from tempor.apis.digitalocean import digitalocean
from tempor.apis.gcp import gcp
from tempor.apis.linode import linode
from tempor.apis.vultr import vultr



#!/usr/bin/env python3

from typing import Dict
import requests
import json


API_URL = 'https://api.vultr.com/v2'

class vultr:

    def get_account(token: str) -> Dict:
        return requests.get(f'{API_URL}/account', headers={
                    'Authorization': f'Bearer {token}',
                    'Content-Type': 'application/json'
                        }).json()
    

    @staticmethod 
    def authorized(token: str) -> bool:
        resp = vultr.get_account(token)
    
        if 'error' in resp:
            return False
    
        return True
    
    
    @staticmethod 
    def get_images(token: str) -> Dict:
        images = dict()

        cursor = ''
        while True:
            resp =  requests.get(f'{API_URL}/os{cursor}', headers={
                    'Authorization': f'Bearer {token}',
                    'Content-Type': 'application/json'
                        }).json()


            for image in resp['os']:
                images[image['id']] = image['name']

            if 'links' not in resp['meta'] or not resp['meta']['links']['next']:
                break
            else:
                cursor = '?cursor=' + resp['meta']['links']['next']

        return images
   

    @staticmethod 
    def valid_image_in_region(image: str, region: str, token: str) -> Dict:
        return True  # vultr does not restrict images in regions


    @staticmethod 
    def get_regions(token: str) -> Dict:
        regions = dict()

        cursor = ''
        while True:
            resp = requests.get(f'{API_URL}/regions{cursor}', headers={
                    'Authorization': f'Bearer {token}',
                    'Content-Type': 'application/json'
                        }).json()
            
            for region in resp['regions']:
                regions[region['id']] = f'{region["city"]}, {region["country"]}'

            if 'links' not in resp['meta'] or not resp['meta']['links']['next']:
                break
            else:
                cursor = '?cursor=' + resp['meta']['links']['next']

        return regions



#!/usr/bin/env python3
# -*- coding:utf-8 -*-
"""
Quick and Easy Infrastructure
"""

from appdirs import *
from pathlib import Path
import logging
import logging.config
import site
import sys
import os

ROOT_DIR = os.path.abspath(os.path.dirname(__file__))

APP_NAME = "tempor"
APP_AUTHOR = "wh1t3fox"

with open(os.path.join(os.path.dirname(__file__), "VERSION")) as fr:
    __version__ = fr.read().strip()

CONFIG_DIR = user_config_dir(APP_NAME)
DATA_DIR = user_data_dir(APP_NAME, APP_AUTHOR)

# this should drop bins in the same path as tempor
#BIN_DIR = os.path.dirname(sys.executable)
BIN_DIR = site.getusersitepackages().split('lib')[0] + 'bin'

if not os.path.exists(CONFIG_DIR):
    os.makedirs(CONFIG_DIR)

if not os.path.exists(DATA_DIR):
    os.makedirs(DATA_DIR)

provider_info = dict()



#!/usr/bin/python

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import open_url
import json
import shlex
import tarfile
import os
import os.path
import shutil
import tempfile
import urllib.parse


DOCUMENTATION = '''
---
module: aur
short_description: Manage packages from the AUR
description:
    - Manage packages from the Arch User Repository (AUR)
author:
    - Kewl <xrjy@nygb.rh.bet(rot13)>
options:
    name:
        description:
            - Name or list of names of the package(s) to install or upgrade.

    state:
        description:
            - Desired state of the package.
        default: present
        choices: [ present, latest ]

    upgrade:
        description:
            - Whether or not to upgrade whole system.
        default: no
        type: bool

    use:
        description:
            - The tool to use, 'auto' uses the first known helper found and makepkg as a fallback.
        default: auto
        choices: [ auto, yay, pacaur, trizen, pikaur, aurman, makepkg ]

    extra_args:
        description:
            - Arguments to pass to the tool.
              Requires that the 'use' option be set to something other than 'auto'.
        type: str

    skip_pgp_check:
        description:
            - Only valid with makepkg.
              Skip PGP signatures verification of source file.
              This is useful when installing packages without GnuPG (properly) configured.
              Cannot be used unless use is set to 'makepkg'.
        type: bool
        default: no

    ignore_arch:
        description:
            - Only valid with makepkg.
              Ignore a missing or incomplete arch field, useful when the PKGBUILD does not have the arch=('yourarch') field.
              Cannot be used unless use is set to 'makepkg'.
        type: bool
        default: no

    aur_only:
        description:
            - Limit helper operation to the AUR.
        type: bool
        default: no

    local_pkgbuild:
        description:
            - Only valid with makepkg or pikaur.
              Directory with PKGBUILD and build files.
              Cannot be used unless use is set to 'makepkg' or 'pikaur'.
        type: path
        default: no
notes:
  - When used with a `loop:` each package will be processed individually,
    it is much more efficient to pass the list directly to the `name` option.
'''

RETURN = '''
msg:
    description: action that has been taken
helper:
    the helper that was actually used
'''

EXAMPLES = '''
- name: Install trizen using makepkg, skip if trizen is already installed
  aur: name=trizen use=makepkg state=present
  become: yes
  become_user: aur_builder
'''

def_lang = ['env', 'LC_ALL=C']

use_cmd = {
    'yay': ['yay', '-S', '--noconfirm', '--needed', '--cleanafter'],
    'pacaur': ['pacaur', '-S', '--noconfirm', '--noedit', '--needed'],
    'trizen': ['trizen', '-S', '--noconfirm', '--noedit', '--needed'],
    'pikaur': ['pikaur', '-S', '--noconfirm', '--noedit', '--needed'],
    'aurman': ['aurman', '-S', '--noconfirm', '--noedit', '--needed', '--skip_news', '--pgp_fetch', '--skip_new_locations'],
    'makepkg': ['makepkg', '--syncdeps', '--install', '--noconfirm', '--needed']
}

use_cmd_local_pkgbuild = {
    'pikaur': ['pikaur', '-P', '--noconfirm', '--noedit', '--needed', '--install'],
    'makepkg': ['makepkg', '--syncdeps', '--install', '--noconfirm', '--needed']
}

has_aur_option = ['yay', 'pacaur', 'trizen', 'pikaur', 'aurman']


def package_installed(module, package):
    """
    Determine if the package is already installed
    """
    rc, _, _ = module.run_command(['pacman', '-Q', package], check_rc=False)
    return rc == 0


def check_packages(module, packages):
    """
    Inform the user what would change if the module were run
    """
    would_be_changed = []
    diff = {
        'before': '',
        'after': '',
    }

    for package in packages:
        installed = package_installed(module, package)
        if not installed:
            would_be_changed.append(package)
            if module._diff:
                diff['after'] += package + "\n"

    if would_be_changed:
        status = True
        if len(packages) > 1:
            message = '{} package(s) would be installed'.format(len(would_be_changed))
        else:
            message = 'package would be installed'
    else:
        status = False
        if len(packages) > 1:
            message = 'all packages are already installed'
        else:
            message = 'package is already installed'
    module.exit_json(changed=status, msg=message, diff=diff)


def build_command_prefix(use, extra_args, skip_pgp_check=False, ignore_arch=False, aur_only=False, local_pkgbuild=None):
    """
    Create the prefix of a command that can be used by the install and upgrade functions.
    """
    if local_pkgbuild:
        command = def_lang + use_cmd_local_pkgbuild[use]
    else:
        command = def_lang + use_cmd[use]
    if skip_pgp_check:
        command.append('--skippgpcheck')
    if ignore_arch:
        command.append('--ignorearch')
    if aur_only and use in has_aur_option:
        command.append('--aur')
    if local_pkgbuild and use != 'makepkg':
        command.append(local_pkgbuild)
    if extra_args:
        command += shlex.split(extra_args)
    return command


def install_with_makepkg(module, package, extra_args, skip_pgp_check, ignore_arch, local_pkgbuild=None):
    """
    Install the specified package or a local PKGBUILD with makepkg
    """
    if not local_pkgbuild:
        module.get_bin_path('fakeroot', required=True)
        f = open_url('https://aur.archlinux.org/rpc/?v=5&type=info&arg={}'.format(urllib.parse.quote(package)))
        result = json.loads(f.read().decode('utf8'))
        if result['resultcount'] != 1:
            return (1, '', 'package {} not found'.format(package))
        result = result['results'][0]
        f = open_url('https://aur.archlinux.org/{}'.format(result['URLPath']))
    with tempfile.TemporaryDirectory() as tmpdir:
        if local_pkgbuild:
            shutil.copytree(local_pkgbuild, tmpdir, dirs_exist_ok=True)
            command = build_command_prefix('makepkg', extra_args)
            rc, out, err = module.run_command(command, cwd=tmpdir, check_rc=True)
        else:
            tar = tarfile.open(mode='r|*', fileobj=f)
            tar.extractall(tmpdir)
            tar.close()
            command = build_command_prefix('makepkg', extra_args, skip_pgp_check=skip_pgp_check, ignore_arch=ignore_arch)
            rc, out, err = module.run_command(command, cwd=os.path.join(tmpdir, result['Name']), check_rc=True)
    return (rc, out, err)


def install_local_package(module, package, use, extra_args, local_pkgbuild):
    """
    Install the specified package with a local PKGBUILD
    """
    with tempfile.TemporaryDirectory() as tmpdir:
        shutil.copytree(local_pkgbuild, tmpdir, dirs_exist_ok=True)
        command = build_command_prefix(use, extra_args, local_pkgbuild=tmpdir + '/PKGBUILD')
        rc, out, err = module.run_command(command, check_rc=True)
    return (rc, out, err)


def check_upgrade(module, use):
    """
    Inform user how many packages would be upgraded
    """
    rc, stdout, stderr = module.run_command([use, '-Qu'], check_rc=True)
    data = stdout.split('\n')
    data.remove('')
    module.exit_json(
        changed=len(data) > 0,
        msg="{} package(s) would be upgraded".format(len(data)),
        helper=use,
    )


def upgrade(module, use, extra_args, aur_only):
    """
    Upgrade the whole system
    """
    assert use in use_cmd

    command = build_command_prefix(use, extra_args, aur_only=aur_only)
    command.append('-u')

    rc, out, err = module.run_command(command, check_rc=True)

    module.exit_json(
        changed=not (out == '' or 'nothing to do' in out or 'No AUR updates found' in out),
        msg='upgraded system',
        helper=use,
    )


def install_packages(module, packages, use, extra_args, state, skip_pgp_check, ignore_arch, aur_only, local_pkgbuild):
    """
    Install the specified packages
    """
    if local_pkgbuild:
        assert use in use_cmd_local_pkgbuild
    else:
        assert use in use_cmd

    changed_iter = False

    for package in packages:
        if state == 'present':
            if package_installed(module, package):
                rc = 0
                continue
        if use == 'makepkg':
            rc, out, err = install_with_makepkg(module, package, extra_args, skip_pgp_check, ignore_arch, local_pkgbuild)
        elif local_pkgbuild:
            rc, out, err = install_local_package(module, package, use, extra_args, local_pkgbuild)
        else:
            command = build_command_prefix(use, extra_args, aur_only=aur_only)
            command.append(package)
            rc, out, err = module.run_command(command, check_rc=True)

        changed_iter = changed_iter or not (out == '' or '-- skipping' in out or 'nothing to do' in out)

    message = 'installed package(s)' if changed_iter else 'package(s) already installed'

    module.exit_json(
        changed=changed_iter,
        msg=message if not rc else err,
        helper=use,
        rc=rc,
    )


def make_module():
    module = AnsibleModule(
        argument_spec={
            'name': {
                'type': 'list',
            },
            'state': {
                'default': 'present',
                'choices': ['present', 'latest'],
            },
            'upgrade': {
                'type': 'bool',
            },
            'use': {
                'default': 'auto',
                'choices': ['auto'] + list(use_cmd.keys()),
            },
            'extra_args': {
                'default': None,
                'type': 'str',
            },
            'skip_pgp_check': {
                'default': False,
                'type': 'bool',
            },
            'ignore_arch': {
                'default': False,
                'type': 'bool',
            },
            'aur_only': {
                'default': False,
                'type': 'bool',
            },
            'local_pkgbuild': {
                'default': None,
                'type': 'path',
            },
        },
        mutually_exclusive=[['name', 'upgrade']],
        required_one_of=[['name', 'upgrade']],
        supports_check_mode=True
    )

    params = module.params

    use = params['use']

    if use == 'auto':
        if params['extra_args'] is not None:
            module.fail_json(msg="'extra_args' cannot be used with 'auto', a tool must be specified.")
        use = 'makepkg'
        # auto: select the first helper for which the bin is found
        for k in use_cmd:
            if module.get_bin_path(k):
                use = k
                break

    if use != 'makepkg' and (params['skip_pgp_check'] or params['ignore_arch']):
        module.fail_json(msg="This option is only available with 'makepkg'.")

    if not (use in use_cmd_local_pkgbuild) and params['local_pkgbuild']:
        module.fail_json(msg="This option is not available with '%s'" % use)

    if params['local_pkgbuild'] and not os.path.isdir(params['local_pkgbuild']):
        module.fail_json(msg="Directory %s not found" % (params['local_pkgbuild']))

    if params['local_pkgbuild'] and not os.access(params['local_pkgbuild'] + '/PKGBUILD', os.R_OK):
        module.fail_json(msg="PKGBUILD inside %s not readable" % (params['local_pkgbuild']))

    if params.get('upgrade', False) and use == 'makepkg':
        module.fail_json(msg="The 'upgrade' action cannot be used with 'makepkg'.")

    return module, use


def apply_module(module, use):
    params = module.params

    if params.get('upgrade', False):
        if module.check_mode:
            check_upgrade(module, use)
        else:
            upgrade(module, use, params['extra_args'], params['aur_only'])
    else:
        if module.check_mode:
            check_packages(module, params['name'])
        else:
            install_packages(module,
                             params['name'],
                             use,
                             params['extra_args'],
                             params['state'],
                             params['skip_pgp_check'],
                             params['ignore_arch'],
                             params['aur_only'],
                             params['local_pkgbuild'])


def main():
    module, use = make_module()
    apply_module(module, use)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding:utf-8 -*-
from pathlib import Path
import string
import random
import json


def random_line(f):
    with open(f) as fr:
        lines = fr.read().splitlines()
        return random.choice(lines).replace("'", "").lower()


def random_number(min_val=0, max_val=100):
    return str(random.randint(min_val, max_val))


def random_str(min_val=5, max_val=10):
    return "".join(
        [
            random.choice(string.ascii_lowercase)
            for _ in range(random.randint(min_val, max_val))
        ]
    )


try:
    wordlist = Path("/usr/share/dict/american-english")
    if not wordlist.is_file():
        raise FileNotFoundError
    name = f"{random_line(wordlist)}{random_number()}"
except FileNotFoundError:
    name = f"{random_str()}{random_number()}"

result = {"name": name}
print(json.dumps(result))



#!/usr/bin/env python3
# -*- coding:utf-8 -*-

from python_terraform import *
from rich.console import Console
from rich.table import Table
from rich.progress import track
from pathlib import Path
import argparse
import time
import json
import sys
import re
import os

from tempor import (
    provider_info,
    ROOT_DIR
)
from tempor.apis import (
    aws,
    azure,
    digitalocean,
    gcp,
    linode,
    vultr
)
from tempor.console import console
from tempor.utils import (
    find_hostname,
    get_config,
    get_hosts,
    image_region_choices,
    rm_hosts,
    save_hosts,
    terraform_installed,
)
from tempor.workspaces import *
from tempor.playbook import run_playbook
from tempor.ssh import check_sshkeys, install_ssh_keys


# TODO: cleanup some of this code
def get_args() -> (str, str, argparse.Namespace):

    cfg = get_config()

    if not cfg:
        sys.exit(1)

    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument("--teardown", default=None, help="Name of VPS Image to Tear down")

    # build args for each provider
    subparsers = parser.add_subparsers(dest='provider')
    for entry in cfg['providers']:
        provider = entry['name']
        api_token = entry['api_token']

        provider_info[provider] = dict()

        # validate API creds
        # only call the APIs once
        if not getattr(globals()[provider], 'authorized')(api_token):
            console.print(f"[red bold] Invalid {provider} API Token. Fix or remove provider.")
            sys.exit(1)

        provider_info[provider]['api_token'] = api_token


        prov_parser = subparsers.add_parser(provider, epilog='', add_help=False)
        prov_parser.add_argument(
            "--image",
            metavar = "image",
            #choices = provider_info[provider]['images'].keys(),
            help="Specify the OS Image",
        )
        prov_parser.add_argument(
            "--region",
            metavar = "region",
            #choices = provider_info[provider]['regions'].keys(),
            help="Specify the Region to Host the Image",
        )
        prov_parser.add_argument("-s", "--setup", action="store_true", help="Create VPS'")
        prov_parser.add_argument("-l", "--list", action="store_true", help="List Available VPS'")
        prov_parser.add_argument(
            "-b", "--bare", action="store_true", help="Leave as a Bare Install"
        )
        prov_parser.add_argument(
            "-m", "--minimal", action="store_true", help="Minimal Configuration"
        )
        prov_parser.add_argument('-h', '--help', action='store_true')

    
    args = parser.parse_args()

    if args.teardown:
        return args


    # Check for a default provider
    if 'default' in cfg and cfg['default']:
        default_provider = cfg['default']
    else:
        default_provider = None

    # if default_provider is not chosen, update settings
    if default_provider != args.provider:
        default_provider = args.provider

    # check options for this provider
    for p in cfg['providers']:
        if p['name'] == default_provider:

            if 'image' in p:
                default_image = p['image']
                if not args.image:
                    args.image = default_image

            if 'region' in p:
                default_region = p['region']
                if not args.region:
                    args.region = default_region

            if 'api_token' in p:
                api_token = p['api_token']
                args.api_token = api_token
            break
    else:
        console.print(f"[red bold]{default_provider} is not a supported provider")
        parser.print_help()
        parser.exit(0)
        sys.exit(1)
    
    if args.provider == 'azure':
        provider_info[args.provider]['images'] =  getattr(globals()[args.provider], 'get_images')(api_token, args.region)
    else:
        provider_info[args.provider]['images'] =  getattr(globals()[args.provider], 'get_images')(api_token)
    provider_info[args.provider]['regions'] =  getattr(globals()[args.provider], 'get_regions')(api_token)

    if args.image not in provider_info[args.provider]['images']:
        console.print(f"[red bold]{args.image} is not a supported image")
        parser.print_help()
        image_region_choices(args.provider)
        parser.exit(0)

    if args.region not in provider_info[args.provider]['regions']:
        console.print(f"[red bold]{args.region} is not a supported region")
        parser.print_help()
        image_region_choices(args.provider)
        parser.exit(0)

    valid_image = getattr(globals()[args.provider], 'valid_image_in_region')(args.image, args.region, api_token)
    assert valid_image, f"{args.image} is not available in {args.region}"

    try:
        if args.help:
            image_region_choices(args.provider)
            parser.exit(0)

        # configure some easier options
        if not args.setup and (args.bare or args.minimal):
            args.setup = True

        if args.provider == 'aws':
            args.user = 'user'
        else:
            args.user = 'root'
    except AttributeError as e:  # typically thrown at the args.help
        parser.print_help()
        parser.exit(0)


    return args


def main(args: argparse.Namespace = None, override_teardown: bool = False) -> None:
    if not args:
        args = get_args()

    # if teardown we need to look up some info
    if args.teardown:
        _host = find_hostname(args.teardown)

        if not _host:
            console.print(f"[red bold]{args.teardown} is not a valid hostname")
            return

        _provider, _, _image = _host['workspace'].replace('+', '/').split('_')
        args.provider = _provider
        args.image = _image
        args.region = _host['region']
        args.api_token = provider_info[args.provider]['api_token']

    plan_path = f"{ROOT_DIR}/providers/{args.provider}/files/plans/{args.region}/{args.image}/plan"
    # The name must contain only URL safe characters, and no path separators.
    tf_workspace_name = f'{args.provider}_{args.region}_{args.image}'.replace('/', '+')

    if override_teardown:
        args.teardown = tf_workspace_name
        args.setup = False
    
    plan_parent_path = Path(plan_path).parent.absolute()
    if not os.path.exists(plan_parent_path):
        os.makedirs(plan_parent_path)

    terr_path = terraform_installed()
    if terr_path is None:
        console.print("[red bold]Platform not Supported")
        return

    t = Terraform(
        working_dir=f"{ROOT_DIR}/providers/{args.provider}",
        variables={"api_token": args.api_token},
        terraform_bin_path=terr_path,
    )

    # make sure terraform is initialized
    ret, stdout, stderr = t.init()
    if ret != 0 and stderr:
        console.print("[red bold]Failed during initialization")
        stderr = re.sub(f'(\[\d+m)', r'\033\1', stderr)
        print(stderr)
        return
    
    current_workspace = get_current_workspace(t)

    # Is the current workspace different than the one we should be using?
    # listing is irrelevant for workspaces
    if (current_workspace != tf_workspace_name) and not args.list:
        workspaces = get_all_workspace(t)

        if tf_workspace_name not in workspaces:
            create_new_workspace(t, tf_workspace_name)
        else:
            select_workspace(t, tf_workspace_name)

    elif not (args.teardown or args.list):
        # Only 1 provider/region/image at a time
        console.print(f"[red bold]Provider/Region/Image Combination taken. Chose a different region, image, or provider.")
        return


    # now what do we want to do?
    if args.teardown:
        console.print(f"Tearing down {args.teardown}...", end="", style="bold italic")
        ret, stdout, stderr = t.cmd("apply", "-destroy", "-auto-approve",
                                var={
                                    "api_token": args.api_token,
                                    "image": args.image,
                                    "region": args.region
                                })
        if ret != 0 and stderr:
            stderr = re.sub(f'(\[\d+m)', r'\033\1', stderr)
            print(stderr)

        # switch to default workspace
        ret, stdout, stderr = t.cmd("workspace", "select", "default")
        if ret != 0 and stderr:
            stderr = re.sub(f'(\[\d+m)', r'\033\1', stderr)
            print(stderr)

        # delete old workspace
        ret, stdout, stderr = t.cmd("workspace", "delete", tf_workspace_name)
        if ret != 0 and stderr:
            stderr = re.sub(f'(\[\d+m)', r'\033\1', stderr)
            print(stderr)

        rm_hosts(args.provider, tf_workspace_name)
        console.print("Done.")
        return


    elif args.list:
        all_hosts = get_hosts()

        if provider in all_hosts:
            table = Table(title="Active VPS'")
            table.add_column("VPS Name", style="cyan")
            table.add_column("IP Address", style="magenta")
            table.add_column("Region", style="magenta")
            table.add_column("Image", style="magenta")

            for host in all_hosts[args.provider]:
                for hostname, values in host.items():
                    image = values['workspace'].split('_')[-1]
                    table.add_row(hostname, values['ip'], values['region'], image.replace('+', '/'))
            console.print(table)
        return

    # prevent accidental creations
    if not args.setup:
        return

    # Creates new key pair
    if check_sshkeys(args.provider, args.region, args.image) is False:
        return

    # lets plan the config
    console.print("Preparing Configuration...", end="", style="bold italic")
    ret, stdout, stderr = t.cmd(
            "plan", f"-out={plan_path}", var={
                            "api_token": args.api_token, 
                            "image": args.image,
                            "region": args.region
                        }
    )
    if ret != 0 and stderr:
        # Fix the color escape sequences
        stderr = re.sub(f'(\[\d+m)', r'\033\1', stderr)
        print(stderr)
        main(args, True)  # force teardown
        return
    console.print("Done.")

    # now apply the config
    console.print("Creating VPS...", end="", style="bold italic")
    ret, stdout, stderr = t.cmd("apply", plan_path)
    if ret != 0 and stderr:
        # Fix the color escape sequences
        stderr = re.sub(f'(\[\d+m)', r'\033\1', stderr)
        print(stderr)
        main(args, True)  # force teardown
        return
    console.print("Done.")

    console.print("Configuring SSH Keys...", end="", style="bold italic")
    # Get Hostname and IP Adress
    output = t.output()

    new_hosts = dict()
    # digitalocean
    if "droplet_ip_address" in output:
        for hostname, ip_address in output["droplet_ip_address"]["value"].items():
            new_hosts[hostname] = {
                    'ip': ip_address, 
                    'region': args.region,
                    'workspace': tf_workspace_name
            }
            install_ssh_keys(args.provider, args.region, args.image, hostname, ip_address, args.user)

    # linode, aws
    elif "instance_ip_address" in output:
        for hostname, ip_address in output["instance_ip_address"]["value"].items():
            new_hosts[hostname] = {
                    'ip': ip_address, 
                    'region': args.region,
                    'workspace': tf_workspace_name
            }
            install_ssh_keys(args.provider, args.region, args.image, hostname, ip_address, args.user)

    # vultr
    elif "server_ip_address" in output:
        for hostname, ip_address in output["server_ip_address"]["value"].items():
            new_hosts[hostname] = {
                    'ip': ip_address, 
                    'region': args.region,
                    'workspace': tf_workspace_name
            }
            install_ssh_keys(args.provider, args.region, args.image, hostname, ip_address, args.user)
    console.print("Done.")

    save_hosts(args.provider, new_hosts)

    if not args.bare:
        playbook = 'minimal.yml' if args.minimal else 'main.yml'
        run_playbook(playbook, args.user)

    console.print("\nVPS' now available!\n", style="bold italic green")
    for host in new_hosts:
        console.print(f"ssh {host}", style="magenta")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import shutil
import os

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join("tempor", "VERSION"), "r", encoding="utf-8") as f:
    version = f.read()

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="tempor",
    version=version,
    description="Ephemeral VPS Creation",
    license="GPT3",
    long_description=long_description,
    author="Wh1t3Fox",
    author_email="dev@exploit.design",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Operating System :: Android",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX :: Linux",
        "Environment :: Console",
    ],
    keywords=["VPS"],
    packages=find_packages(exclude=["contrib", "docs", "tests"]),
    package_data={"tempor": ["config", "providers", "playbooks"]},
    install_requires=[
        "ansible==5.6.0",
        "appdirs==1.4.4",
        "ansible_runner==1.4.6",
        "jsonschema==3.2.0",
        "python-terraform==0.10.1",
        "rich==9.3.0",
        "ssh-config==0.0.22",
        "PyYAML==5.4.1",
        "boto3==1.22.1",
        "requests==2.27.1",
        "google-api-python-client==2.46.0"
    ],
    extras_require={
        "dev": [
            "black==20.8b1",
            "twine==3.2.0",
            "pytest==6.1.2",
            "pytest_cov==2.10.1",
            "flit==3.0.0",
        ],
    },
    entry_points={
        "console_scripts": [
            "tempor = tempor.__main__:main",
        ],
    },
    include_package_data=True,
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import ansible_runner
import os

from tempor import ROOT_DIR, DATA_DIR
from tempor.console import console


def run_playbook(playbook: str = "main.yml") -> None:
    ansible_data = f"{DATA_DIR}/ansible"
    if not os.path.exists(ansible_data):
        os.makedirs(ansible_data)

    ansible_runner.run(
        private_data_dir=f"{ROOT_DIR}/playbooks", playbook=playbook, verbosity=5
    )
